import JSZip from "jszip";
import axios from "axios";
import { saveAs } from "file-saver";
import Vue from 'vue';
import {
    getSubjectImageZipInfo,
} from "@/api/trials/visit.js";
export const downloadImage = async (id) => {
    try {
        let res = await getSubjectImageZipInfo(id);
        if (res.IsSuccess) {
            let item = res.Result;
            await setfolder(item);
        }
    } catch (err) {
        console.log(err);
    }
};
const setfolder = async (item) => {
    const zip = new JSZip(); // 创建实例对象
    let zipObj = {};
    const promises = [];
    for (let patient of item.PatientList) {
        if (!zipObj[patient.PatientIdStr]) {
            zipObj[patient.PatientIdStr] = zip.folder(
                `${item.SubjectCode}_${patient.PatientIdStr}`
            );
        }
        for (let visit of patient.VisitList) {
            if (!zipObj[`${patient.PatientIdStr}${visit.VisitName}`]) {
                zipObj[`${patient.PatientIdStr}${visit.VisitName}`] = zipObj[
                    patient.PatientIdStr
                ].folder(visit.VisitName);
            }
            for (let study of visit.StudyList) {
                let date = study.StudyTime.split(" ")[0];
                for (let series of study.SeriesList) {
                    if (
                        !zipObj[
                        `${patient.PatientIdStr}${visit.VisitName}${series.Modality}`
                        ]
                    ) {
                        zipObj[
                            `${patient.PatientIdStr}${visit.VisitName}${series.Modality}`
                        ] = zipObj[`${patient.PatientIdStr}${visit.VisitName}`].folder(
                            `${date}_${series.Modality}`
                        );
                    }
                    for (let instance of series.InstancePathList) {
                        let obj = {
                            subjectCode: item.SubjectCode,
                            patientIdStr: patient.PatientIdStr,
                            visitName: visit.VisitName,
                            date: study.StudyTime.split(" ")[0],
                            modality: series.Modality,
                            instancePath: instance.Path,
                            dicomName: instance.Path.split("/Dicom/")[1],
                        };
                        const promise = handleBatchDown(
                            obj,
                            zipObj[
                            `${patient.PatientIdStr}${visit.VisitName}${series.Modality}`
                            ]
                        );
                        promises.push(promise);
                    }
                }
            }
        }
    }
    // 生成 zip 文件
    Promise.all(promises)
        .then(() => {
            // 生成zip 文件
            zip
                .generateAsync({
                    type: "blob",
                    compression: "DEFLATE", // STORE: 默认不压缩， DEFLATE：需要压缩
                    compressionOptions: {
                        level: 9, // 压缩等级 1~9   1 压缩速度最快， 9 最优压缩方式
                    },
                })
                .then((res) => {
                    saveAs(res, item.SubjectCode + new Date().getTime() + "_CV.zip"); // 使用FileSaver.saveAs保存文件，文件名可自定义
                    zipObj = null;
                });
        })
        .catch((reason) => { });
};
const handleBatchDown = async (item, zip) => {
    return new Promise((resolve) => {
        getFileData(
            Vue.prototype.OSSclientConfig.basePath + item.instancePath
        ).then((res) => {
            const fileName = item.dicomName + ".dcm";
            zip.file(fileName, res.data, { binary: true });
            resolve();
        });
    });
};
const getFileData = (fileUrl) => {
    return new Promise((resolve, reject) => {
        axios(fileUrl, {
            method: "GET",
            responseType: "blob", // 返回的数据会被强制转为blob类型 ，转换成arraybuffer 也行
        })
            .then((res) => {
                resolve(res);
            })
            .catch((error) => {
                reject(error);
            });
    });
};